##############################################################################################################
##                                                                                                          ##
## This application extracts mysql and loads to AWS S3 folder                                               ##
## It takes two parameter:                                                                                  ##
##    1. AWS secret name for the getting the connection string to connect to the mysql database             ##
##    2. Config file name that has the database, table names and the location of the S3 bucket              ##
##      Usage : python3 mysql_extract.py secret_name config_file                                            ##
##                                                                                                          ##
##      Author :  Raju Pillai    Date : April 20, 2024                                                      ##
##############################################################################################################

import pandas as pd
import boto3
import json
import os
from sqlalchemy import create_engine
from datetime import datetime
import pyarrow as pa
import pyarrow.parquet as pq

def fetch_secret(secret_name):
    client = boto3.client('secretsmanager')
    response = client.get_secret_value(SecretId=secret_name)
    return json.loads(response['SecretString'])

def read_config(config_file):
    import yaml
    with open(config_file, 'r') as file:
        return yaml.safe_load(file)

def connect_db(connection_string):
    engine = create_engine(connection_string)
    return engine

def write_to_parquet(dataframe, filename):
    table = pa.Table.from_pandas(dataframe)
    pq.write_table(table, filename)

def main(secret_name, config_file):
    # Fetch secrets and config
    secret = fetch_secret(secret_name)
    config = read_config(config_file)
    
    # Database connection
    connection_string = f"mysql+pymysql://{secret['username']}:{secret['password']}@{secret['host']}:{secret['port']}/{config['database']}"
    engine = connect_db(connection_string)

    # Prepare S3
    s3_client = boto3.client('s3')
    date_folder = datetime.now().strftime('%Y-%m-%d')

    # Log file setup
    log_filename = f"log_{date_folder}.txt"
    with open(log_filename, 'w') as log_file:
        # Write header for the log
        log_file.write(f"{'Start Time':<20} {'End Time':<20} {'Rows':<10} {'S3 Path':<60}\n")
        log_file.write('-' * 110 + '\n')
        
        for table in config['tables']:
            table_folder = os.path.join(config['s3_folder'], date_folder, table)

            start_time = datetime.now()
            df = pd.read_sql_table(table, con=engine)
            end_time = datetime.now()
            
            # Write to parquet
            os.makedirs(table_folder, exist_ok=True)  # Ensure the directory exists
            parquet_file_path = os.path.join(table_folder, f"{table}.parquet")
            write_to_parquet(df, parquet_file_path)
            
            # Upload to S3
            s3_path = os.path.join(table_folder, f"{table}.parquet")
            s3_client.upload_file(parquet_file_path, config['s3_bucket'], s3_path)
            
            # Logging in a tabular format
            log_entry = f"{start_time.strftime('%Y-%m-%d %H:%M:%S'):<20} {end_time.strftime('%Y-%m-%d %H:%M:%S'):<20} {len(df):<10} s3://{config['s3_bucket']}/{s3_path}\n"
            log_file.write(log_entry)

            # Remove the local parquet file
            os.remove(parquet_file_path)

            # Remove the local directory if empty
            if not os.listdir(table_folder):
                os.rmdir(table_folder)
    rm_folder = os.path.join(config['s3_folder'], date_folder)
    print("Removing folder: " + rm_folder)
    os.rmdir(rm_folder)
if __name__ == "__main__":
    import sys
    secret_name = sys.argv[1]
    config_file = sys.argv[2]
    main(secret_name, config_file)

